#include <stdio.h>
#include <stdlib.h>
#include <gsl/gsl_blas.h>

void gsl_my_print_matrix(gsl_matrix * m);
void gsl_my_print_vector(gsl_vector * v);
void gsl_print_matrix_view_elements(gsl_matrix_view mv);

void gsl_blas_level1_example();
void gsl_blas_level2_example();
void gsl_blas_level3_example();

int main()
{
     printf("\n *** BLAS level 1***");
     gsl_blas_level1_example();
     printf("\n *** BLAS level 2***");
     gsl_blas_level2_example();
     printf("\n *** BLAS level 3***");
     gsl_blas_level3_example();
     return 0;
}

void gsl_blas_level1_example(){
    float alpha;
    alpha=2.0;
    gsl_vector * x = gsl_vector_alloc (3);
    gsl_vector_set (x, 0, 3);
    gsl_vector_set (x, 1, 5);
    gsl_vector_set (x, 2, 7);
     printf("\n[x] \n");
     gsl_my_print_vector(x);
    gsl_vector * y = gsl_vector_alloc (3);
    gsl_blas_daxpy(alpha, x , y );
    printf("\n BLAS level 1: y = a x where (a=%5.2f)\n",alpha);
    gsl_my_print_vector(y);

     gsl_vector_free (x);
     gsl_vector_free (y);
}

void gsl_blas_level2_example(){
     double a[] = { 1, 2, 3, 4 };
     gsl_matrix_view A = gsl_matrix_view_array(a, 2, 2);
     printf("\n[A] ");
     gsl_print_matrix_view_elements(A);
     gsl_vector * x = gsl_vector_alloc (2);
     gsl_vector_set (x, 0, 1);
     gsl_vector_set (x, 1, 1);
      printf("\n\n[x] \n");
     gsl_my_print_vector(x);
     gsl_vector * y = gsl_vector_alloc (2);

    gsl_blas_dgemv(CblasNoTrans,1.0,&A.matrix,x,0.0,y);
    printf("\n BLAS level 2: y = a Ax +by where (a=1.0 and b=0.0)\n");
    gsl_my_print_vector(y);
    gsl_vector_free (x);
    gsl_vector_free (y);
}

void gsl_blas_level3_example(){
    int rows,columns;
    rows=3;
    columns=3;
    gsl_matrix * A = gsl_matrix_alloc (rows, columns);
    gsl_matrix * B = gsl_matrix_alloc (rows, columns);
    gsl_matrix * C = gsl_matrix_alloc (rows, columns);
    gsl_matrix_set_all(A, 3.0);
    gsl_matrix_memcpy (B, A);
    printf("[A] ");
    gsl_my_print_matrix(A);

    printf("\n\n [B]");
    gsl_my_print_matrix(B);

    gsl_blas_dgemm (CblasNoTrans, CblasNoTrans,1.0, A, B, 0.0, C);
    printf("\n\n BLAS level 3: C = a AB +bC where (a=1.0 and b=0.0)");
    gsl_my_print_matrix(C);
    gsl_matrix_free (A);
    gsl_matrix_free (B);
    gsl_matrix_free (C);
}

void gsl_my_print_vector(gsl_vector * v) {
     int i;
     for (i = 0; i < v->size; i++)
                   printf ("[%d =] %g\n", i, gsl_vector_get (v, i));
}

void gsl_my_print_matrix(gsl_matrix * m)
{
    int i, j;
    printf("\t\t");
    for (i = 0; i < m->size1; i++)
    {
        printf("\n");
        for (j = 0; j < m->size2; j++)
            printf ("[%d,%d)] = %g\t", i, j, gsl_matrix_get (m, i, j));
    }
}

void gsl_print_matrix_view_elements(gsl_matrix_view mv){
        int i, j;

        for (i = 0; i < mv.matrix.size1; i++){
            printf("\n");
            for (j = 0; j < mv.matrix.size2; j++)
                    printf ("[%d,%d] = %g\t", i, j, gsl_matrix_get (&mv.matrix, i, j));
         }
}
