#include <stdio.h>
#include <stdlib.h>
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_blas.h>
void gsl_my_print_matrix(gsl_matrix * m);
void gsl_my_transpose_matrix(gsl_matrix * ma);
void gsl_my_add_matrix_elements(gsl_matrix * ma,gsl_matrix * mb);
void gsl_my_sub_matrix_elements(gsl_matrix * ma,gsl_matrix * mb);
void gsl_my_mul_matrix_elements(gsl_matrix * ma,gsl_matrix * mb);
void gsl_my_div_matrix_elements(gsl_matrix * ma,gsl_matrix * mb);
void gsl_my_matrix_scale(gsl_matrix * m, double scale);
void gsl_my_matrix_add_constant(gsl_matrix * m, double constant);

const int rows=3;
const int columns=3;
int main()
{
    gsl_matrix * A = gsl_matrix_alloc (rows, columns);
    gsl_matrix * B = gsl_matrix_alloc (rows, columns);

    gsl_matrix_set_all(A, 3.0);
    gsl_matrix_memcpy (B, A);

    printf("[A] ");
    gsl_my_print_matrix(A);

    printf("\n\n [B]");
    gsl_my_print_matrix(B);

    gsl_my_matrix_scale(B, 10.0);
    gsl_my_add_matrix_elements(A,B);
    gsl_my_sub_matrix_elements(A,B);
    gsl_my_mul_matrix_elements(A,B);
    gsl_my_div_matrix_elements(A,B);
    gsl_my_matrix_add_constant(B, 1.5);
    gsl_my_transpose_matrix(A);

    return 0;
}

void gsl_my_matrix_scale(gsl_matrix * m, double scale)
{
    printf("\n\n [B]  scale %10.2f", scale);
    gsl_matrix *tmp = gsl_matrix_alloc (rows, columns);
    gsl_matrix_memcpy (tmp, m);
    gsl_matrix_scale(tmp, scale);
    gsl_my_print_matrix(tmp);
    gsl_matrix_free (tmp);
}

void gsl_my_matrix_add_constant(gsl_matrix * m, double constant)
{
    printf("\n\n [B]  add constant %10.2f", constant);
    gsl_matrix *tmp = gsl_matrix_alloc (rows, columns);
    gsl_matrix_memcpy (tmp, m);
    gsl_matrix_add_constant(tmp, constant);
    gsl_my_print_matrix(tmp);
    gsl_matrix_free (tmp);
}


void gsl_my_add_matrix_elements(gsl_matrix * ma,gsl_matrix * mb)
{
    printf("\n\n gsl_matrix_add(A, B);");
    gsl_matrix *tmp = gsl_matrix_alloc (rows, columns);
    gsl_matrix_memcpy (tmp, ma);
    gsl_matrix_add(tmp, mb);
    gsl_my_print_matrix(tmp);
    gsl_matrix_free (tmp);
}


void gsl_my_sub_matrix_elements(gsl_matrix * ma,gsl_matrix * mb)
{
    printf("\n\n gsl_matrix_sub(A, B);");
    gsl_matrix *tmp = gsl_matrix_alloc (rows, columns);
    gsl_matrix_memcpy (tmp, ma);
    gsl_matrix_sub(tmp, mb);
    gsl_my_print_matrix(tmp);
    gsl_matrix_free (tmp);
}



void gsl_my_mul_matrix_elements(gsl_matrix * ma,gsl_matrix * mb)
{
    printf("\n\n gsl_matrix_mul(A, B);");
    gsl_matrix *tmp = gsl_matrix_alloc (rows, columns);
    gsl_matrix_memcpy (tmp, ma);
    gsl_matrix_mul_elements(tmp, mb);
    gsl_my_print_matrix(tmp);
    gsl_matrix_free (tmp);
}

void gsl_my_div_matrix_elements(gsl_matrix * ma,gsl_matrix * mb)
{
    printf("\n\n gsl_matrix_div(A, B);");
    gsl_matrix *tmp = gsl_matrix_alloc (rows, columns);
    gsl_matrix_memcpy (tmp, ma);
    gsl_matrix_div_elements(tmp, mb);
    gsl_my_print_matrix(tmp);
    gsl_matrix_free (tmp);
}

void gsl_my_transpose_matrix(gsl_matrix * ma)
{
    printf("\n\n gsl_matrix_transpose ");
    gsl_matrix *tmp = gsl_matrix_alloc (rows, columns);
    gsl_matrix_memcpy (tmp, ma);
    gsl_matrix_transpose (tmp);
    gsl_my_print_matrix(tmp);
    gsl_matrix_free (tmp);
}

void gsl_my_print_matrix(gsl_matrix * m)
{
    int i, j;
    printf("\t\t");
    for (i = 0; i < m->size1; i++)
    {
        printf("\n");
        for (j = 0; j < m->size2; j++)
            printf ("[%d,%d)] = %g\t", i, j, gsl_matrix_get (m, i, j));
    }
}
